/*
 * @author Gabriel Balestrin
 */
package Commands;

import Robo.Ponto;
import java.util.*;
import virbot4u.CenaX3D;
import virbot4u.JanelaPonto;
import virbot4u.JanelaPrograma;

public abstract class AbstractBlockContext extends Thread implements BlockContext {

    protected BlockContext parent;
    private Map<String, InstrLabel> labels;
    private Map<String, BlockContext> routines;
    private Map<String, Float> variables;
    private int label;
    protected CenaX3D scene;
    boolean loop;
    boolean upDown;
    private long timeStart;
    Instruction interruptInstr;

    public AbstractBlockContext(BlockContext parent,
            Map<String, InstrLabel> labels,
            Map<String, BlockContext> routines,
            Map<String, Float> variables,
            CenaX3D scene) {
        this.variables = variables;
        this.parent = parent;
        this.labels = labels;
        this.routines = routines;
        this.label = 0;
        this.scene = scene;
        this.loop = false;
        this.upDown = false;
        this.timeStart = System.currentTimeMillis();
        this.interruptInstr = null;

    }

    public BlockContext getParentBlockContext() {
        return parent;
    }

    /** Adds instruction to the end of this block */
    protected abstract void addInstr(Instruction instr);

    public void addInstrOG(int line) {
        addInstr(new InstrOG(line));
    }

    public void addInstrCG(int line) {
        addInstr(new InstrCG(line));
    }

    public void addInstrWT(String time, int line) {
        addInstr(new InstrWT(time, line));
    }

    public void addInstrRB(int line) {
        addInstr(new InstrRB(line));
    }

    public void addInstrGP(ArrayList position, int approach, int line) {
        addInstr(new InstrGP(position, approach, line));
    }

    public void addInstrJU(String toLabel, int line) {
        addInstr(new InstrJU(toLabel, line));
    }

    public void addInstrCS(String sub, int line) {
        addInstr(new InstrCS(sub, line));
    }

    public void addInstrSVC(Nodo variable, Nodo value, int line) {
        addInstr(new InstrSVC(variable, value, line));
    }

    public void addInstrSVS(String nameVar, int line) {
        addInstr(new InstrSVS(nameVar, line));
    }

    public void addInstrSVT(String nameVar, int line) {
        addInstr(new InstrSVT(nameVar, line));
    }

    public void addInstrIF(Nodo condition, String toLabel, int line) {
        addInstr(new InstrIF(condition, toLabel, line));
    }

    public void addInstrPS(String msg, int line) {
        addInstr(new InstrPS(msg, line));
    }

    public void addInstrRP(String namePoint, int line) {
        addInstr(new InstrRP(namePoint, line));
    }

    public void addInstrJA(Nodo value, int line) {
        addInstr(new InstrJA(value, line));
    }

    public void addInstrRT(int line) {
        addInstr(new InstrRT(line));
    }

    public void addInstrIL(Nodo axis, String toLabel, int line) {
        addInstr(new InstrIL(axis, toLabel, line));
    }

    public void addInstrPH(int line) {
        addInstr(new InstrPH(line));
    }

    public void addInstrII(Nodo iiToken, boolean onOff, String toLabel, int line) {
        addInstr(new InstrII(iiToken, onOff, toLabel, line));
    }

    public void addInstrOI(Nodo ioToken, boolean onOff, String toLabel, int line) {
        addInstr(new InstrOI(ioToken, onOff, toLabel, line));
    }

    public void addInstrDI(Nodo inOut, int line) {
        addInstr(new InstrDI(inOut, line));
    }

    public void addInstrEI(Nodo inOut, int line) {
        addInstr(new InstrEI(inOut, line));
    }

    public void addInstrOF(Nodo inOut, int line) {
        addInstr(new InstrOF(inOut, line));
    }

    public void addInstrON(Nodo inOut, int line) {
        addInstr(new InstrON(inOut, line));
    }

    public void addInstrGC(String positionVia, String positionF, int approach, int line) {
        addInstr(new InstrGC(positionVia, positionF, approach, line));
    }

    public void addInstrGL(ArrayList position, int approach, int line) {
        addInstr(new InstrGL(position, approach, line));
    }

    public void addInstrSA(String axis, int line) {
        addInstr(new InstrSA(axis, line));
    }

    public void addInstrGS(ArrayList position, int approach, int line) {
        addInstr(new InstrGS(position, approach, line));
    }

    public void createVariable(String varName, Float value) {
        this.variables.put(varName, value);
    }

    public Float getVariable(String varName) {
        return this.variables.get(varName);
    }

    public long getTimeStart() {
        return timeStart;
    }

    public void initTimeStart() {
        timeStart = System.currentTimeMillis();
    }

    public boolean isVariable(String varName) {
        if (this.variables.get(varName) == null) {
            return false;
        }
        return true;
    }

    public void createLabel(String labelName, int line) {
        InstrLabel li = new InstrLabel(labelName, line);
        if (labels.get(labelName) != null) {
            informError(ErrorMessages.ERROR_MULTIPLES_LABEL, line, labelName);
            this.stop();
        }
        labels.put(labelName, li);
        addInstr(li);
    }

    public Instruction getInstrForLabel(String labelName) {
        return labels.get(labelName);
    }

    public boolean executeRoutine(String routineName) {
        if (routines.get(routineName) == null) {
            return false;
        }
        routines.get(routineName).executeBlockInstructions();
        return true;
    }

    public BlockContext createRoutine(String routineName) {
        BlockContext bc = new ScriptingBlockContext(this, this.scene);
        System.out.println("--------" + routineName);

        routines.put(routineName, bc);
        return bc;
    }

    public int informLabel() {
        return this.label++;
    }

    public CenaX3D getScene() {
        return this.scene;
    }

    public void setExecutionLoop() {
        loop = true;
    }

    public void informError(int errorMessage, int lineError, String complements) {
        JanelaPrograma.getTxtConsole().setCaretPosition(0);
        if (JanelaPrograma.getIdioma().equals("Portugues")) {
            JanelaPrograma.getTxtConsole().setText((JanelaPrograma.getTxtConsole().getText().equals("")) ? "" + ErrorMessages.getMessage(errorMessage, lineError, ErrorMessages.PORTUGUESE, complements) + "\n" : JanelaPrograma.getTxtConsole().getText() + ErrorMessages.getMessage(errorMessage, lineError, ErrorMessages.PORTUGUESE, complements) + "\n");
            scene.jp.setTxtFeedback("ERRO na linha << " + lineError + " >>");
        } else {
            JanelaPrograma.getTxtConsole().setText((JanelaPrograma.getTxtConsole().getText().equals("")) ? "" + ErrorMessages.getMessage(errorMessage, lineError, ErrorMessages.ENGLISH, complements) + "\n" : JanelaPrograma.getTxtConsole().getText() + ErrorMessages.getMessage(errorMessage, lineError, ErrorMessages.ENGLISH, complements) + "\n");
            scene.jp.setTxtFeedback("ERROR line << " + lineError + " >>");
        }
        this.stop();
    }

    public void informWarning(int warningMessage, int lineError, String complements) {
        JanelaPrograma.getTxtConsole().setCaretPosition(0);
        if (JanelaPrograma.getIdioma().equals("Portugues")) {
            JanelaPrograma.getTxtConsole().setText(ErrorMessages.getMessage(warningMessage, lineError, ErrorMessages.PORTUGUESE, complements));
        } else {
            JanelaPrograma.getTxtConsole().setText(ErrorMessages.getMessage(warningMessage, lineError, ErrorMessages.ENGLISH, complements));
        }
    }

    public void printMessageConsole(String textMessage) {
        JanelaPrograma.getTxtConsole().setText((JanelaPrograma.getTxtConsole().getText().equals("")) ? "" + textMessage + "\n" : JanelaPrograma.getTxtConsole().getText() + textMessage + "\n");
        JanelaPrograma.getTxtConsole().setCaretPosition(JanelaPrograma.getTxtConsole().getText().length());

    }

    public synchronized void setInterruption(boolean upDown, int input, Instruction instr, int line, int contr) {
        this.upDown = upDown;
        System.out.println(upDown);
        if (contr == 1) {
            writeFeedback(3, "" + line);
        }
        this.interruptInstr = instr;
    }

    public int verificaPontos(String nomePonto, int line) {
        Ponto ponto;

        if (JanelaPonto.getListaDePontos() == null) {
            informError(ErrorMessages.ERROR_POINTLIST, line, "");
        }

        Integer numPontos = JanelaPonto.getListaDePontos().getItemCount();
        for (int i = 0; i < numPontos; i++) {
            ponto = (Ponto) JanelaPonto.getListaDePontos().getObjeto(i);
            if (nomePonto.equals(ponto.getNomePonto())) {
                return i;
            }
        }
        return -1;
    }

    /* Used to GUI*/
    protected void writeFeedbackLine(int line) {
        if (JanelaPrograma.getIdioma().equals("Portugues")) {
            scene.jp.setTxtFeedback("Executando linha << " + line + " >> ...");
        } else {
            scene.jp.setTxtFeedback("Executing line << " + line + " >> ...");
        }
    }

    /* Used to GUI*/
    protected void writeFeedback(int type, String complements) {
        switch (type) {
            case 1:
                if (JanelaPrograma.getIdioma().equals("Portugues")) {
                    scene.jp.setTxtFeedback("Terminado...");
                } else {
                    scene.jp.setTxtFeedback("Finished...");
                }
                break;
            case 2:
                if (JanelaPrograma.getIdioma().equals("Portugues")) {
                    JanelaPrograma.getTxtConsole().setText(JanelaPrograma.getTxtConsole().getText() + "Completo!");
                } else {
                    JanelaPrograma.getTxtConsole().setText(JanelaPrograma.getTxtConsole().getText() + "Complete!");
                }
                break;
            case 3:
                if (JanelaPrograma.getIdioma().equals("Portugues")) {
                    JanelaPrograma.getTxtConsole().setText(JanelaPrograma.getTxtConsole().getText() + "Executando Interrupção na linha '" + complements + "'!\n");
                } else {
                    JanelaPrograma.getTxtConsole().setText(JanelaPrograma.getTxtConsole().getText() + "Executing Interruption in line '" + complements + "'!\n");
                }
                break;
        }
    }
}
