/*
 * @author Gabriel Balestrin
 */
package Commands;

import java.util.ArrayList;
import virbot4u.*;
/**
 * Contains all the information about a block
 * (a block can be a subroutine body or the body of
 * the whole program (instructions outside subroutines)).
 */
public interface BlockContext {

	/** Add "OG" instruction at the end of the block */
	void addInstrOG(int line);
	/** Add "CG" instruction at the end of the block */
	void addInstrCG(int line);
        /** Add "CG" instruction at the end of the block */
	void addInstrWT(String time,int line);
        /** Add "CG" instruction at the end of the block */
	void addInstrGP(ArrayList position,  int approach, int line);
        /** Add "CG" instruction at the end of the block */
	void addInstrRB(int line);
	/** Add "JU" instruction at the end of the block */
	void addInstrJU(String toLabel,int line);
	/** Add "CS" instruction at the end of the block */
	void addInstrCS(String routineName,int line);
        /** Add "CS" instruction at the end of the block */
	void addInstrSVC(Nodo variable, Nodo value,int line);
	/** Add "IF" instruction at the end of the block */
	void addInstrIF(Nodo condition, String toLabel,int line);
        /** Add "PS" instruction at the end of the block */
        void addInstrPS(String msg,int line);
        /** Add "RP" instruction at the end of the block */
        void addInstrRP(String namePoint,int line);
        /** Add "SVS" instruction at the end of the block */
        void addInstrSVS(String nameVar,int line);
        /** Add "SVT" instruction at the end of the block */
        void addInstrSVT(String nameVar,int line);
        /** Add "JA" instruction at the end of the block */
        void addInstrJA(Nodo value, int line);
        /** Add "SVT" instruction at the end of the block */
        void addInstrRT(int line);
        /** Add "IL" instruction at the end of the block */
        void addInstrIL(Nodo axis,String toLabel, int line);
        /** Add "PH" instruction at the end of the block */
        void addInstrPH(int line);
        /** Add "II" instruction at the end of the block */
        void addInstrII(Nodo iiToken, boolean onOff, String toLabel, int line);
        /** Add "II" instruction at the end of the block */
        void addInstrOI(Nodo iiToken, boolean onOff, String toLabel, int line);
        /** Add "EI" instruction at the end of the block */
        void addInstrEI(Nodo inOut, int line);
        /** Add "DI" instruction at the end of the block */
        void addInstrDI(Nodo inOut, int line);
        /** Add "ON" instruction at the end of the block */
        void addInstrON(Nodo inOut, int line);
        /** Add "OF" instruction at the end of the block */
        void addInstrOF(Nodo inOut, int line);
        /** Add "GL" instruction at the end of the block */
        void addInstrGL(ArrayList position, int approach, int line);
        /** Add "GC" instruction at the end of the block */
        void addInstrGC(String positionVia, String positionF , int approach, int line);
        /** Add "SA" instruction at the end of the block */
        void addInstrSA(String axis, int line);
        /** Add "GS" instruction at the end of the block */
        void addInstrGS(ArrayList position, int approach,  int line);

        /* Insert a variable in a HashMap*/
        void createVariable(String varName, Float value);

        /* Return a variable value*/
        Float getVariable(String varName);

        /* Return true if there is a variable*/
        boolean isVariable(String varName);

        /* Return the time that executing program beginned*/
        long getTimeStart();

        /* Init the time */
        void initTimeStart();

        /** Creates a label at the end of the block */
	void createLabel(String labelName,int line);

        /** Returns the instruction that follows the label "labelName" */
	Instruction getInstrForLabel(String labelName);

        /** Returns the instruction that follows the routine "sub" */
	boolean executeRoutine(String routineName);

        /** Creates and returns the context of a new routine */
	BlockContext createRoutine(String routineName);

        /** Returns BlockContext of outer scope */
	BlockContext getParentBlockContext();

        /** Execute added instructions, starting at current position */
	void executeBlockInstructions();

        /** Create and show a message error! */
        void informError(int errorMessage, int lineError, String complements);
       
        /** Create and show a message error! */
        void informWarning(int errorMessage, int lineError, String complements);

        /* Print a message in the console*/
        void printMessageConsole(String textMessage);

        /** Return a avaliable label*/
        int informLabel();

        /** Return a scena X3D*/
        CenaX3D getScene();
       
        /** Execute a Thread compiler*/
        void start();

        /** Analizes about the thread's life*/
        boolean isAlive();

        /** Stop execution of the thread*/
        void stop();

        /* Execute one next instruction of compiler*/
        void executeNextInstruction();

        /* Set a loop boolean in true*/
        void setExecutionLoop();

        /* Set a variable Line in 0*/
        void inicializeProgram();
  
        /** Add a interruption */
        void setInterruption(boolean upDown, int input, Instruction instr, int line, int contr);

        /** Get point in the point list*/
        int verificaPontos(String nomePonto, int line);
}


