/*
 * @author Gabriel Balestrin
 */
package Commands;

import Componentes.RList;
import Robo.Angulos;
import Robo.Ponto;
import java.util.ArrayList;
import virbot4u.JanelaPonto;

public class InstrGL extends AbstractInstruction {

    private ArrayList position;
    private RList lPontos;
    private int approach;

    public InstrGL(ArrayList position, int approach, int line) {
        this.position = position;
        this.line = line;
        this.approach = approach;
    }

    public Instruction execute(BlockContext inThisContext) {
        float granularidade = inThisContext.getScene().granularity;
        float h = 1 / granularidade;
        float xCurrent, yCurrent, zCurrent;
        float x, y, z;
        float roll, pitch;
        Angulos ang;
        ArrayList angs, points = null;
        Ponto pnt = null;
        int cont;
        int sleep;

        /* granularity limit*/
        if(granularidade > 1000)
            granularidade = 1000;
        if(granularidade < 5)
            granularidade = 5;

        /* Get point List */
        lPontos = JanelaPonto.getListaDePontos();
        if (lPontos == null) {
            inThisContext.informError(ErrorMessages.ERROR_POINTLIST, line, "");
        }

        /*GL p1...3 --> [p1, 3, #]*/
        if (position.size() == 3 && position.get(2).equals("#")) {
            int cont1, cont2;
            String pointName1, pointName2;
            pointName1 = position.get(0).toString();
            pointName2 = position.get(1).toString();

            /* Remove the last point and the simbol '#'*/
            position.remove(2);
            position.remove(1);

            /* Looking for the right variable*/
            if (!Character.isDigit(pointName1.charAt(0))) {
                cont1 = inThisContext.verificaPontos(pointName1,line);
            } else {
                cont1 = Integer.parseInt(pointName1) - 1;
            }
            if (!Character.isDigit(pointName2.charAt(0))) {
                cont2 = inThisContext.verificaPontos(pointName2,line);
            } else {
                cont2 = Integer.parseInt(pointName2) - 1;
            }

            if (cont1 > cont2) {
                inThisContext.informError(ErrorMessages.ERROR_POINTSEQUENCE, line, pointName1 + " ... " + pointName2);
            }

            for (int i = cont1 + 1; i <= cont2; i++) {
                position.add(((Ponto) lPontos.getObjeto(i)).getNomePonto());
            }
            position.add(pointName2);
        }

        String pointName;
        for (int i = 0; i < position.size(); i++) {
            if (position.get(i).toString().length() > 0) {
                pointName = position.get(i).toString();

                /* If point name, else index of list*/
                if (!Character.isDigit(pointName.charAt(0))) {
                    cont = inThisContext.verificaPontos(pointName,line);
                } else {
                    cont = Integer.parseInt(pointName) - 1;
                }

                if (cont == -1 || cont > lPontos.getItemCount() - 1) {
                    inThisContext.informError(ErrorMessages.ERROR_POINTPOSITION, line, (cont == -1 ? position.get(i).toString() + " " : ""));
                }
                // get the current point
                pnt = (Ponto) lPontos.getObjeto(cont);

                if (inThisContext.getScene() == null) {
                    inThisContext.informError(ErrorMessages.ERROR_SCENE, line, "");
                } else {
                    
                // calculate approach fixed
                float auxRoll = inThisContext.getScene().toRadianos(inThisContext.getScene().getRotacao());

                    //APROACH move
                    this.setApproach(inThisContext, approach, line);
                    try { Thread.sleep(35); } catch (InterruptedException e) {}

                    // get current position
                    xCurrent = inThisContext.getScene().getPontoX();
                    yCurrent = inThisContext.getScene().getPontoY();
                    zCurrent = inThisContext.getScene().getPontoZ();

                    // point list of interpolations points
                    points = new ArrayList();

                    float x1,x2,y1,y2,z1,z2;
                    x1=x2=y1=y2=z1=z2=0;
                    
                    // calculate the new points
                    for (int p = 0; p <= granularidade; p++) {

                        x = xCurrent + (h * p) * (pnt.getX() - xCurrent);
                        y = yCurrent + (h * p) * (pnt.getY() - yCurrent);
                        z = zCurrent + (h * p) * (pnt.getZ() - zCurrent);
                        //get first point
                        if(p == 0){
                            x1 = x;
                            y1 = y;
                            z1 = z;
                        }
                        //get last point
                        if(p == granularidade){
                            x2 = x;
                            y2 = y;
                            z2 = z;
                        }

                        pitch = inThisContext.getScene().toRadianos(inThisContext.getScene().getElevacao() + inThisContext.getScene().getBeta() + inThisContext.getScene().getGama()) - (float) Math.PI;
                        roll = inThisContext.getScene().toRadianos(inThisContext.getScene().getRotacao());

                        angs = inThisContext.getScene().calcularInversaPontos(-x, z, y, pitch, roll, inThisContext.getScene().elbowOver);
                        if (angs == null) {
                            inThisContext.informError(ErrorMessages.ERROR_GLINVALID, line, (String) position.get(i));
                        }
                        ang = new Angulos(inThisContext.getScene().toGraus((Float) angs.get(0)), inThisContext.getScene().toGraus((Float) angs.get(1)), inThisContext.getScene().toGraus((Float) angs.get(2)), inThisContext.getScene().toGraus((Float) angs.get(3)), inThisContext.getScene().toGraus((Float) angs.get(4)));
                        points.add(ang);
                    }
                                                                     // ________________________
                    //point distance between first and last point d = |/ (x2-x1)+(y2-y1)+(z2-z1)
                    double distance = Math.sqrt( Math.pow( x2 - x1, 2 ) + Math.pow( y1 - y2, 2 ) + Math.pow( z1 - z2, 2 ));
                    
                    sleep = (int)(granularidade * distance)/1000;

                    for (int p = 0; p <= granularidade; p++) {
                        inThisContext.getScene().setAngulos((Angulos) points.get(p));                        
                        try { Thread.sleep(sleep); } catch (InterruptedException e) {}

                    }
                }

            }

        }
        return nextInstruction;
    }
}


