/*
 * @author Rafael Pacheco
 */

package Commands;

import Robo.Ponto;
import java.util.ArrayList;
import java.util.Vector;

public class InstrGS extends AbstractInstruction {

    private ArrayList gsPoints;
    private int approach;

    public InstrGS(ArrayList gsPoints, int approach, int line) {
        this.gsPoints = gsPoints;
        this.line = line;
        this.approach = approach;
    }

    public Instruction execute(BlockContext inThisContext) {
        float valApproach = 0;
        boolean isFree = false;
        Vector<Ponto> smoothPoints = new Vector<Ponto>();

        // calculate approach fixed
        float auxRoll = inThisContext.getScene().toRadianos(inThisContext.getScene().getRotacao());

        //set the aproach
        switch(approach){
            case 0: // default approach FREE
                isFree = true;
            break;
            case 1://free
                isFree = true;
            break;
            case 2://front
                valApproach = -1.5707989f;
            break;
            case 3://up
                valApproach = 0.0f;
            break;
            case 4://fixed
                valApproach = inThisContext.getScene().toRadianos(inThisContext.getScene().getElevacao() + inThisContext.getScene().getBeta() + inThisContext.getScene().getGama()) - (float) Math.PI;
            break;
            case 5://back
                valApproach = -4.712389f;
            break;
            default:
                inThisContext.informError(ErrorMessages.ERROR_APPROACH, line, "");
        }


        /*GS p1...3 --> [p1, 3, #]*/
        if (gsPoints.size() == 3 && gsPoints.get(2).equals("#")) {
            int cont1 = -1, cont2 = -1;
            String pointName1, pointName2;
            pointName1 = gsPoints.get(0).toString();
            pointName2 = gsPoints.get(1).toString();

            /* remove the last one and the simbol '#'*/
            gsPoints.remove(2);
            gsPoints.remove(1);
            
            /* look for the right variable*/
            if (!Character.isDigit(pointName1.charAt(0))) {
                for (int j = 0; j < inThisContext.getScene().jp.janelaPonto.lControlPoints.getItemCount(); j++) {
                    Ponto p = (Ponto) inThisContext.getScene().jp.janelaPonto.lControlPoints.getObjeto(j);
                    if (p.getNomePonto().compareTo(pointName1) == 0)
                        cont1 = j;
                        break;
                }
                if(cont1 == -1)
                    inThisContext.informError(ErrorMessages.ERROR_POINT_CONTROL, line, pointName1);
            } else {
                cont1 = Integer.parseInt(pointName1) - 1;
            }
            if (!Character.isDigit(pointName2.charAt(0))) {
                for (int j = 0; j < inThisContext.getScene().jp.janelaPonto.lControlPoints.getItemCount(); j++) {
                    Ponto p = (Ponto) inThisContext.getScene().jp.janelaPonto.lControlPoints.getObjeto(j);
                    if (p.getNomePonto().compareTo(pointName2) == 0)
                        cont2 = j;
                        break;
                }
                if(cont2 == -1)
                    inThisContext.informError(ErrorMessages.ERROR_POINT_CONTROL, line, pointName2);
            } else {
                cont2 = Integer.parseInt(pointName2) - 1;
            }

            if (cont1 > cont2) {
                inThisContext.informError(ErrorMessages.ERROR_POINTSEQUENCE, line, pointName1 + " ... " + pointName2);
            }

            for (int i = cont1 + 1; i < cont2; i++) {
                gsPoints.add(((Ponto) inThisContext.getScene().jp.janelaPonto.lControlPoints.getObjeto(i)).getNomePonto());
            }
            gsPoints.add(pointName2);
        }


        // At least 4 points
        if (gsPoints.size() < 4)
            inThisContext.informError(ErrorMessages.ERROR_POINT_NUMBER_GS, line, "");
        

        for (int i = 0; i < gsPoints.size(); i++) {
           boolean found = false;
           for (int j = 0; j < inThisContext.getScene().jp.janelaPonto.lControlPoints.getItemCount(); j++) {
               Ponto p = (Ponto) inThisContext.getScene().jp.janelaPonto.lControlPoints.getObjeto(j);
               if (p.getNomePonto().compareTo((String)gsPoints.get(i)) == 0 || ((String)gsPoints.get(i)).equals(new String(""+(j+1)))) {
                   smoothPoints.add(p);
                   found = true;
                   break;
                }
             }
             if (found == false) {
                inThisContext.informError(ErrorMessages.ERROR_POINT_CONTROL, line, (String)gsPoints.get(i));
             }
             found = false;
         }
        
        Vector<Ponto> pontos = inThisContext.getScene().getSmoothInterpolation(false, smoothPoints);

        // granularity to the movimentation
        int sleep = (int) (500 / inThisContext.getScene().granularity);
        if (sleep < 1) {
            sleep = 1;
        }

        for (int i = 0; i < pontos.size(); i++) {
            Ponto p = pontos.elementAt(i);

            if (isFree == true) {
                valApproach = p.radianPitch;
            }

            inThisContext.getScene().calcularInversa(-p.getX(), p.getZ(), p.getY(), valApproach, isFree == true ? p.radianRoll : auxRoll, inThisContext.getScene().elbowOver); // abordagem fixa

            if (inThisContext.getScene().getPosicaoOK() == false) 
                inThisContext.informError(ErrorMessages.ERROR_POINT_GSM, line, "");
                        
            try {
                Thread.sleep(sleep);
            } catch (InterruptedException e) {
            }
        }

        pontos.clear();

        return nextInstruction;
    }
}
