/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Preensao;

/**
 *
 * @author Cleber
 */
import Objetos.Cilindro;
import Objetos.Esfera;
import Objetos.ListaQuadrante;
import Objetos.Paralelepipedo;

public class TrataPreensao {

	public static final int APROVADO = 1;
	public static final int ERRO_BOUNDING_BOX_TRABALHO = -1;
	public static final int ERRO_FATIA_TRASEIRA = -2;
	public static final int ERRO_TAMANHO_OBJETO_GARRA = -3;
	public static final int ERRO_PROXIMIDADE = -4;
	public static final int ERRO_ABORDAGEM = -5;
	public static final int ERRO_FG_Z = -6;
	public static final int ERRO_FG_X = -7;
	public static final int ERRO_FG_Y = -8;

	public static int preensao(ListaQuadrante lista,int id,Garra g,double deltaAb,double piDeltaGe) {
        char obj;
		//O objeto é uma Esfera
                if(lista.tipoComponente(id).startsWith("class Objetos.E"))
			obj = 'E';
		//O objeto é um Cilindro
		else if (lista.tipoComponente(id).startsWith("class Objetos.C"))
			obj = 'C';
		//O objeto é um Paralelepipedo
		else
			obj = 'P';

		int ret = exclusao(lista,id,obj,g);
		if(ret<0)
			return ret;

		return confirmacao(lista,id,obj,g,deltaAb,piDeltaGe);
	}

	//Etapa da Exclusão
	private static int exclusao(ListaQuadrante lista,int id,char obj,Garra g) {
		if(!boundingBoxTrabalho(lista,id))
			return ERRO_BOUNDING_BOX_TRABALHO;

		if(!fatiaTraseira(lista,id))
			return ERRO_FATIA_TRASEIRA;

		if(!tamanhoObjetoGarra(lista,id,g))
			return ERRO_TAMANHO_OBJETO_GARRA;

		if(!proximidade(lista,id,obj,g))
			return ERRO_PROXIMIDADE;

		return APROVADO;
	}

	//Teste de Bounding Box de Trabalho
	private static boolean boundingBoxTrabalho(ListaQuadrante lista,int id) {
		//Leva-se em consideração a escala do Robô
		if(lista.getObbTreePos(id).getRoot().getCentro()[0]<-12.7*0.075||lista.
                getObbTreePos(id).getRoot().getCentro()[0]>12.7*0.075)
			return false;
		if(lista.getObbTreePos(id).getRoot().getCentro()[2]<-12.7*0.075||lista.
                getObbTreePos(id).getRoot().getCentro()[2]>12.7*0.075)
			return false;
		return true;
	}

	//Teste de Fatia Traseira do Robô
	private static boolean fatiaTraseira(ListaQuadrante lista,int id) {
		double angulo = Math.atan2(lista.getObbTreePos(id).getRoot().
              getCentro()[2],-lista.getObbTreePos(id).getRoot().getCentro()[0]);
		//Conversão para ângulos positivos
		if(angulo<0)
			angulo = 2*Math.PI + angulo;
		//Se é necessário um ângulo maior do que o máximo permitido pela base
		if(angulo>(31*Math.PI)/18) //310 graus
			return false;
		return true;
	}

	//Teste de Tamanho do Objeto
	private static boolean tamanhoObjetoGarra(ListaQuadrante lista,int id,Garra g) {
		if(Math.min(lista.getObbTreePos(id).getRoot().getExtensao()[0],Math.min
                (lista.getObbTreePos(id).getRoot().getExtensao()[1],
                lista.getObbTreePos(id).getRoot().getExtensao()[2]))>Garra.O)
			return false;
		return true;
	}

	//Proximidade
	private static boolean proximidade(ListaQuadrante lista,int id,char obj,Garra g) {
		double dist;
        double R;

		if(obj == 'E') {
			Esfera e = lista.getComponenteEsfera(id);
			dist = distanciaPontoPonto(e.getObbTree().getRoot().getCentro(),g.getPFP());
			R = e.getRaioEsferaEnv()*e.getRaioEsferaEnv();
		}
		else if(obj == 'C') {
			Cilindro c = lista.getComponenteCilindro(id);
			dist = distanciaPontoPonto(c.getObbTree().getRoot().getCentro(),g.getPFP());
			R = c.getRaioEsferaEnv()*c.getRaioEsferaEnv();
		}
		else {
			Paralelepipedo p = lista.getComponenteParalelepipedo(id);
			dist = distanciaPontoPonto(p.getObbTree().getRoot().getCentro(),g.getPFP());
			R = p.getRaioEsferaEnv()*p.getRaioEsferaEnv();
		}

		//É eliminado caso a distância seja maior do que o raio da esfera envolvente
		if(dist>R)
			return false;

		return true;
	}

	//Etapa da Confirmação
	private static int confirmacao(ListaQuadrante lista,int id,char obj,Garra g,double deltaAb,double piDeltaGe) {
		int ret = folgaAbordagem(lista,id,obj,g,deltaAb);
		if(ret<0)
			return ret;

        return folgaGeometrica(lista,id,obj,g,piDeltaGe);
	}

	//Análise de Folga de Abordagem
	private static int folgaAbordagem(ListaQuadrante lista,int id,char obj,Garra g,double deltaAb) {
		//Teste de Folga de Abordagem para a Geometria Cilindro
		if(obj == 'C')
			return folgaAbordagemCilindro(lista.getComponenteCilindro(id),g,deltaAb);
		//Teste de Folga de Abordagem para a Geometria Paralelepípedo
		else if(obj == 'P')
			return folgaAbordagemParalelepipedo(lista.getComponenteParalelepipedo(id),g,deltaAb);
		//Não é necessário teste de Abordagem para a Geometria Esfera
		return APROVADO;
	}

	//Análise de Folga de Abordagem para Cilindro
	private static int folgaAbordagemCilindro(Cilindro c,Garra g,double deltaAb) {
		//Cálculo do ângulo entre o vetor Up do Cilindro e o vetor de Aproximação da Garra
		double angulo = anguloEntreVetores(c.getObbTree().getRoot().getEixoY(),g.getApg());
		//A garra está pegando o Cilindro por cima
		if(angulo<=deltaAb||Math.PI - angulo<=deltaAb)
			//Não é necessário testes com outros vetores, pois independe o rolamento existente na garra para esta situação
			return APROVADO;
		else if(angulo>=Math.PI/2-deltaAb&&angulo<=Math.PI/2+deltaAb) {
			//Cálculo do ângulo entre o vetor Up do Cilindro e o vetor Up da Garra
			angulo = anguloEntreVetores(g.getUpg(),c.getObbTree().getRoot().getEixoY());
			//A garra está segurando o cilindro pelas suas laterais
			if(angulo<=deltaAb||Math.PI - angulo<=deltaAb)
				return APROVADO;
			//A garra está segurando o cilindro pelas suas faces circulares
			else if(angulo>=Math.PI/2-deltaAb&&angulo<=Math.PI/2+deltaAb)
				return APROVADO;
			//A garra não pode pegar o cilindro
			else
				return ERRO_ABORDAGEM;
		}
		//A garra está pegando o cilindro pelo lado de maneira inclinada
		else {
			//Cálculo do ângulo entre o vetor Up do cilindro e o vetor El da Garra
			angulo = anguloEntreVetores(c.getObbTree().getRoot().getEixoY(),g.getElg());
			//A garra está alinhada com a lateral do cilindro
			if(angulo>=Math.PI/2-deltaAb&&angulo<=Math.PI/2+deltaAb)
				return APROVADO;
			//A garra não pode pegar o cilindro
			else
				return ERRO_ABORDAGEM;
		}
	}

	//Análise de Folga de Abordagem para Paralelepípedo
	private static int folgaAbordagemParalelepipedo(Paralelepipedo p,Garra g,double deltaAb) {
		double angulo;

        //Verifica se existe paralelismo entre o vetor Eixo Lateral da garra com qualquer vetor de orientação do paralelepípedo
		if((angulo = anguloEntreVetores(p.getObbTree().getRoot().getEixoX(),g.getElg()))<=
                deltaAb||Math.PI-angulo<=deltaAb)
			return APROVADO;
		else if((angulo = anguloEntreVetores(p.getObbTree().getRoot().getEixoY(),g.getElg()))<=
                deltaAb||Math.PI-angulo<=deltaAb)
			return APROVADO;
		else if((angulo = anguloEntreVetores(p.getObbTree().getRoot().getEixoZ(),g.getElg()))<=
                deltaAb||Math.PI-angulo<=deltaAb)
			return APROVADO;
		else
			return ERRO_ABORDAGEM;
	}

	//Análise de Folga Geométrica
	private static int folgaGeometrica(ListaQuadrante lista,int id,char obj,Garra g,double piDeltaGe) {
		double[] centro;

		if(obj == 'E') {
			Esfera e = lista.getComponenteEsfera(id);
			centro = new double[] {e.getPosicao()[0],e.getPosicao()[1]+0.48,e.getPosicao()[2]};
		}
		else if (obj == 'C') {
			Cilindro c = lista.getComponenteCilindro(id);
			centro = new double[] {c.getPosicao()[0],c.getPosicao()[1]+0.48,c.getPosicao()[2]};
		}
		else {
			Paralelepipedo p = lista.getComponenteParalelepipedo(id);
			centro = new double[] {p.getPosicao()[0],p.getPosicao()[1]+0.48,p.getPosicao()[2]};
		}
		return testeFolgaGeometrica(centro,g,piDeltaGe);
	}
    //Método que calcula se a distância do ponto central do objeto com o PFP
	private static int testeFolgaGeometrica(double[] pontoCentralFolga,Garra g,double deltaGe) {
        if(Math.abs(pontoCentralFolga[0]-g.getPFP()[0])>deltaGe && Math.abs(pontoCentralFolga[0]-
                g.getPFP()[0])>=Math.abs(pontoCentralFolga[1]-g.getPFP()[1]) && Math.
                abs(pontoCentralFolga[0]-g.getPFP()[0])>=Math.abs(pontoCentralFolga[2]-g.getPFP()[2]))
            return ERRO_FG_X;
		if(Math.abs(pontoCentralFolga[1]-g.getPFP()[1])>deltaGe && Math.abs(pontoCentralFolga[1]-
                g.getPFP()[1])>=Math.abs(pontoCentralFolga[0]-g.getPFP()[0]) && Math.abs(pontoCentralFolga[1]-
                g.getPFP()[1])>=Math.abs(pontoCentralFolga[2]-g.getPFP()[2]))
            return ERRO_FG_Y;
		if(Math.abs(pontoCentralFolga[2]-g.getPFP()[2])>deltaGe && Math.abs(pontoCentralFolga[2]-
                g.getPFP()[2])>=Math.abs(pontoCentralFolga[0]-g.getPFP()[0]) && Math.abs(pontoCentralFolga[2]-
                g.getPFP()[2])>=Math.abs(pontoCentralFolga[1]-g.getPFP()[1]))
            return ERRO_FG_Z;
        return APROVADO;
	}

	//Método que calcula a distância entre dois pontos
	private static double distanciaPontoPonto(double[] a,double[] b) {
		return Math.pow(a[0]-b[0],2) + Math.pow((a[1]+0.48)-b[1],2) + Math.pow(a[2]-b[2],2);
	}

	//Método que calcula o ângulo entre dois vetores
	private static double anguloEntreVetores(double v1[],double v2[]) {
		return Math.acos((v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2])/(Math.sqrt(v1[0]*v1[0] + v1[1]*v1[1] + v1[2]*v1[2])*Math.sqrt(v2[0]*v2[0] + v2[1]*v2[1] + v2[2]*v2[2])));
	}
}
